
# q-expansion of newform 810.4.c.b, downloaded from the LMFDB on 14 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 810
weight = 4
poly_data = [5604552, -7064280, 4452100, 60664, 20930, -26726, 17389, 326, 8, -4, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[10845982033193988, -13820528928725072, -937022557772180, -817010462860, 39705915326929, -54085095296996, -1055859400495, -14292291865, 9839161208, -3069306257], 5970220296345000], [[7171446374939268, 38282298444612, -6778906066992, -24011410104392, 52991266291077, -1234618288671, -29990085728, -9319640328, 3338428935, -126984661], 128391834330000], [[397138179818268, 15547020659136, -2263092712104, -10469755817624, 2678026285335, -584083598376, -10310201786, -1158090966, 256633707, -36511750], 25678366866000], [[-97615490736404988, 15085082420518628, -135988439680688, -653944713433388, -282015397440667, 131746660558111, 2584536858358, 25567729558, -34071112675, 7693431431], 3980146864230000], [[204721175549294892, -307532425376078864, -9196851427341272, -3553296163883212, 613546394150779, -1326608333184854, -24761276608498, -514785758458, 224040810959, -74929422752], 23880881185380000], [[4748594341258080, 13252041373231508, 210715574433416, -289844243501324, -1401329722204, 52761410611535, 1027047802894, -7371870266, -7163257166, 2948768567], 770351005980000], [[-696006476977351356, 889492890100767160, -25937440091767928, 2327996974467152, -2549230466993531, 3340215353329042, 59856351456698, 1113969161918, -614441663143, 188930110468], 23880881185380000], [[-768854255982764400, 583546323243322036, 16940800113592792, -385611921564988, -2909262446145128, 1718493460137415, 33188487895928, 673324404908, -406287078952, 98781710869], 11940440592690000], [[-173796622910356716, 146936033781721568, 1422633930610168, -692264577086292, -610908606399275, 601840148832682, 11460236117462, 145374880822, -118550902779, 34320285080], 2653431242820000]]

hecke_ring_character_values = [[731, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [487, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, -1, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [2, 0, 0, 0, 0, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 1, 0, 0], [-3, 0, 0, 1, 0, 0, 0, 0, 0, 0], [-1, -2, 0, 0, 0, -2, 1, -1, 0, 1], [-1, -2, 0, 0, -2, 0, -1, 0, 0, 1], [-6, 0, 1, -2, -2, -3, 1, 0, -1, 0], [-1, 6, 0, 0, 1, -3, 2, 3, 0, 1], [21, 0, -1, -1, -3, 3, 3, 0, 0, 0], [10, 0, 0, -3, -1, 6, 2, 0, 1, 0], [2, 14, 0, 0, 1, 0, -1, 2, 1, -3], [3, 0, 4, 0, -3, 13, 5, 0, 2, 0], [2, 0, 0, 0, -4, 11, -6, -4, -1, -1], [3, -28, 0, 0, 7, -16, 4, 9, 5, -8], [-5, 74, 0, 0, -9, -16, -4, -9, 5, 0], [-213, 0, 1, -2, -15, 5, 13, 0, -2, 0], [101, 0, 16, -2, 4, -9, -5, 0, -1, 0], [-7, 20, 0, 0, -11, -24, -4, 0, 7, 0], [-279, 0, 12, -7, 12, 8, -8, 0, 4, 0], [-1, 200, 0, 0, -14, 18, -13, 6, -2, 3], [-147, 0, -8, -8, 15, 0, -12, 0, 3, 0], [5, 112, 0, 0, -1, 5, -6, -6, 2, -7], [138, 0, -9, 9, 30, 25, -19, 0, 11, 0], [-5, 268, 0, 0, -19, -15, -14, -14, 8, -3], [420, 0, 17, -9, -3, 38, 10, 0, 7, 0], [6, -102, 0, 0, -24, 9, -30, 19, 9, -15], [-8, -6, 0, 0, -32, -14, -24, 0, 10, -2], [-157, 0, 7, 7, 39, -9, -33, 0, 6, 0], [5, 88, 0, 0, 36, -20, 31, -6, -2, -3], [0, -114, 0, 0, -24, 42, -24, -53, -6, 6], [-162, 0, 3, 8, -36, -19, 25, 0, -11, 0], [-28, -354, 0, 0, -7, -1, 21, 18, -16, 44], [-13, 0, -1, 2, -11, -69, -5, 0, -16, 0], [15, 0, -24, 40, -6, 51, 15, 0, 9, 0], [388, 0, -12, 27, 35, 120, -4, 0, 31, 0], [10, -416, 0, 0, -31, 117, -41, -77, -22, 12], [31, 604, 0, 0, -7, 63, -38, 18, 2, -33], [28, -656, 0, 0, 26, 21, -2, -54, 3, -31], [-1, 562, 0, 0, 34, -36, 35, 27, 0, 1], [783, 0, 29, 0, 39, -9, -33, 0, 6, 0], [1154, 0, 25, 19, -1, -24, -4, 0, -5, 0], [2673, 0, -24, 11, -30, -70, 10, 0, -20, 0], [5, -20, 0, 0, 24, 76, 19, 16, -30, 25], [15, -632, 0, 0, -24, -36, -39, 75, 30, -45], [564, 0, -20, -2, -14, -126, -14, 0, -28, 0], [1740, 0, -3, 6, 4, -9, -5, 0, -1, 0], [-7, -568, 0, 0, 49, -174, 56, -60, 37, -30], [30, -1140, 0, 0, -20, 47, -50, 60, 11, -41], [751, 0, -8, 4, 58, -33, -53, 0, 5, 0], [33, 476, 0, 0, 60, -30, 27, -120, 12, -45], [510, 0, 68, -78, -36, -94, 10, 0, -26, 0], [796, 0, -57, -63, -24, 129, 45, 0, 21, 0], [-3516, 0, -37, 13, -69, 89, 73, 0, 4, 0], [-102, -1084, 0, 0, -73, -197, 29, -84, 22, 80], [-50, 246, 0, 0, -48, -139, 2, -159, 29, 21], [2157, 0, -23, -35, -135, 45, 117, 0, -18, 0], [2533, 0, 82, -38, 13, -138, -38, 0, -25, 0], [51, 1990, 0, 0, 79, -16, 28, 87, 13, -64], [3933, 0, -63, -9, 69, 186, -18, 0, 51, 0], [-46, 944, 0, 0, -62, -78, -16, -130, 16, 30], [12, 750, 0, 0, -81, -3, -93, 39, 36, -48], [89, 424, 0, 0, 75, 193, -14, -16, -30, -59], [-1296, 0, -42, 145, 201, 154, -130, 0, 71, 0], [-110, -1214, 0, 0, -67, 21, 43, -56, -58, 168], [-32, -924, 0, 0, 23, -393, 55, 63, 102, -70], [1951, 0, -64, 95, -96, -84, 60, 0, -36, 0], [40, -4122, 0, 0, -8, -14, -48, 124, 34, -74], [34, -734, 0, 0, -100, 177, -134, 108, -3, -31], [308, 0, -9, 153, -61, -54, 38, 0, -23, 0], [-53, 82, 0, 0, -51, -97, 2, 126, 14, 39], [3615, 0, -8, -31, -114, 214, 134, 0, 20, 0], [2, 3768, 0, 0, 38, -94, 36, 172, 20, -22], [67, 2884, 0, 0, 221, -147, 154, 236, 20, -87], [109, 0, 33, 63, -4, -321, -61, 0, -65, 0], [-119, 196, 0, 0, -177, -175, -58, -177, 38, 81], [-1494, 0, 58, 12, 0, 120, 24, 0, 24, 0], [99, 936, 0, 0, 270, -117, 171, 52, 15, -114], [1407, 0, -3, 15, 141, -56, -124, 0, 17, 0], [463, 0, 1, -77, -275, -240, 172, 0, -103, 0], [-4476, 0, -100, -52, 120, -80, -112, 0, 8, 0], [3913, 0, 139, -221, -59, -81, 31, 0, -28, 0], [-4278, 0, 150, 109, -33, -282, -30, 0, -63, 0], [18, -2318, 0, 0, 13, -313, -5, 322, 112, -130], [-300, 0, -44, -143, -219, -36, 168, 0, -51, 0], [27, 752, 0, 0, 41, 199, 14, -168, -62, 35], [4575, 0, 110, 76, 45, 125, -11, 0, 34, 0], [5, 4066, 0, 0, 114, 166, 109, -318, -90, 85], [1770, 0, 163, -123, -171, -304, 76, 0, -95, 0], [-82, 3468, 0, 0, 104, -49, 186, -201, -73, 155], [83, -3342, 0, 0, 137, -103, 54, -216, 44, -127], [-231, 0, 108, -41, -6, 56, 16, 0, 10, 0], [-180, 2542, 0, 0, -122, 29, 58, 209, -89, 269], [2448, 0, 117, -166, -156, -49, 115, 0, -41, 0], [3606, 0, -77, 40, 164, 171, -97, 0, 67, 0], [32, 1342, 0, 0, -24, 436, -56, 216, -116, 84], [-5496, 0, 128, 114, -48, -482, -58, 0, -106, 0], [-2193, 0, 171, -287, -90, 185, 109, 0, 19, 0], [127, -1944, 0, 0, -173, 334, -300, -224, 31, -158], [-3377, 0, -159, 93, 215, 765, -19, 0, 196, 0], [-112, 5526, 0, 0, -172, 32, -60, 226, -28, 140], [105, -4904, 0, 0, -140, 200, -245, 531, 50, -155], [177, -194, 0, 0, 71, 955, -106, 138, -224, 47], [915, 0, -352, 106, 90, -270, -126, 0, -36, 0], [-5012, 0, -16, -31, -31, 336, 92, 0, 61, 0], [-87, 2846, 0, 0, -382, -410, -295, -160, 206, -119], [-190, -2824, 0, 0, -116, 159, 74, 42, -141, 331], [-96, 996, 0, 0, -225, 105, -129, -678, -24, 120], [-2172, 0, -194, 225, -123, -112, 76, 0, -47, 0], [-451, 0, -106, 128, 340, 0, -272, 0, 68, 0], [-196, -7062, 0, 0, -406, -199, -210, 206, 71, 125], [-194, -1060, 0, 0, 120, -643, 314, -285, 45, 149], [48, -3628, 0, 0, 207, 501, 159, 150, -204, 156], [11551, 0, -29, -143, -28, 93, 41, 0, 13, 0], [-9502, 0, 70, 7, 537, -252, -480, 0, 57, 0], [-7176, 0, -273, 95, 180, 715, -1, 0, 179, 0], [93, 4226, 0, 0, -37, -449, -130, -68, 224, -317], [-15, -1862, 0, 0, -383, 680, -368, -108, -109, 124], [-76, 1258, 0, 0, 212, -334, 288, 150, -10, 86], [342, 0, 233, -121, -309, -1021, 43, 0, -266, 0], [-6319, 0, -97, -55, 403, -723, -467, 0, -64, 0], [212, -3248, 0, 0, 225, 415, 13, 584, -72, -140], [254, -6390, 0, 0, 120, 133, -134, -423, 85, -339], [130, 3336, 0, 0, -330, 296, -460, 186, 98, -228], [-6290, 0, -87, 69, 509, -519, -511, 0, -2, 0], [3039, 0, 15, 119, 705, 535, -457, 0, 248, 0], [6519, 0, 280, -224, -162, -1053, -81, 0, -243, 0], [10611, 0, 478, -191, 198, -668, -292, 0, -94, 0], [103, 8424, 0, 0, 469, 13, 366, -111, -92, -11], [-350, -390, 0, 0, -122, 148, 228, 418, -242, 592], [11762, 0, -250, 227, 665, -180, -568, 0, 97, 0], [257, 4450, 0, 0, 567, 187, 310, -102, -80, -177], [-9247, 0, 362, -250, -167, 702, 274, 0, 107, 0], [-392, -340, 0, 0, -438, -1147, -46, -415, 267, 125], [4881, 0, -208, -102, -126, 946, 290, 0, 164, 0], [18156, 0, -77, 79, 452, -177, -397, 0, 55, 0], [-30, 8634, 0, 0, -285, -198, -255, 6, 141, -111], [188, -7894, 0, 0, 298, -1095, 110, 302, 391, -579], [93, -7386, 0, 0, 100, -235, 7, -480, 107, -200], [8433, 0, -426, -18, 474, 1146, -150, 0, 324, 0], [-8036, 0, 300, -201, -49, -816, -124, 0, -173, 0], [-17841, 0, -163, -417, -315, -385, 175, 0, -140, 0], [-462, 2302, 0, 0, -194, -1552, 268, -268, 274, 188], [89, -9812, 0, 0, -467, 654, -556, -126, -3, -86], [8838, 0, -183, 489, -227, 672, 316, 0, 89, 0], [2169, 0, -154, -253, -246, -784, 40, 0, -206, 0], [157, -1524, 0, 0, -605, 1087, -762, 256, -56, -101], [-32, 138, 0, 0, 283, -1097, 315, 432, 250, -218], [-518, 0, -422, 25, -129, 1464, 396, 0, 267, 0], [166, 1936, 0, 0, 122, 444, -44, 1251, -78, -88], [-255, -4596, 0, 0, -432, -168, -177, -811, 30, 225], [72, 0, -135, 105, -429, -486, 246, 0, -183, 0], [263, -158, 0, 0, 333, 1039, 70, -736, -282, 19], [490, 1296, 0, 0, 286, 772, -204, -273, -26, -464], [144, 5850, 0, 0, -792, 1152, -936, -348, -24, -120], [-11703, 0, -302, 69, 144, -454, -206, 0, -62, 0], [14169, 0, -80, 295, 470, -120, -400, 0, 70, 0], [7620, 0, 511, -133, 372, 643, -169, 0, 203, 0], [-10, -2248, 0, 0, 1003, -1359, 1013, -222, 112, -102], [21975, 0, -558, 150, 306, 719, -101, 0, 205, 0], [-17, -5600, 0, 0, -541, 912, -524, 438, -135, 152], [10, -948, 0, 0, -67, 267, -77, -300, -60, 50], [-152, -3014, 0, 0, -226, 105, -74, 238, -61, 213], [10494, 0, 171, -612, -1068, 183, 891, 0, -177, 0], [-4, -5758, 0, 0, -32, -984, -28, 222, 336, -332], [735, 1532, 0, 0, 129, 1251, -606, 390, 30, -765], [-11945, 0, 24, 75, -840, -840, 504, 0, -336, 0], [-358, -4688, 0, 0, 447, -713, 805, -133, -150, 508]]
